@@ -38,11 +38,11 @@
 #include <TelepathyQt/Constants>
 #include <TelepathyQt/ContactCapabilities>
 
-#include <KTp/Models/accounts-model-item.h>
-#include <KTp/Models/accounts-filter-model.h>
-#include <KTp/Models/contact-model-item.h>
 #include <KTp/presence.h>
 #include <KTp/global-presence.h>
+#include <KTp/actions.h>
+#include <KTp/contact-factory.h>
+#include <KTp/contact.h>
 
 struct MatchInfo {
     Tp::AccountPtr account;
@@ -55,7 +55,6 @@ Q_DECLARE_METATYPE(MatchInfo);
 
 ContactRunner::ContactRunner(QObject *parent, const QVariantList &args):
     Plasma::AbstractRunner(parent, args),
-    m_accountsModel(new AccountsModel(this)),
     m_globalPresence(new KTp::GlobalPresence(this))
 {
     Q_UNUSED(args);
@@ -114,7 +113,7 @@ ContactRunner::ContactRunner(QObject *parent, const QVariantList &args):
                                                     << Tp::Connection::FeatureSelfContact
                                                     << Tp::Connection::FeatureRoster);
 
-    Tp::ContactFactoryPtr contactFactory = Tp::ContactFactory::create(
+    Tp::ContactFactoryPtr contactFactory = KTp::ContactFactory::create(
                                                 Tp::Features()  << Tp::Contact::FeatureAlias
                                                     << Tp::Contact::FeatureAvatarData
                                                     << Tp::Contact::FeatureSimplePresence
@@ -142,7 +141,6 @@ void ContactRunner::accountManagerReady(Tp::PendingOperation *operation)
 
     kDebug() << "Accounts manager is ready!";
 
-    m_accountsModel->setAccountManager(m_accountManager);
     m_globalPresence->setAccountManager(m_accountManager);
 }
 
@@ -228,35 +226,15 @@ void ContactRunner::run(const Plasma::RunnerContext &context, const Plasma::Quer
     Tp::AccountPtr account = data.account;
     Tp::ContactPtr contact = data.contact;
 
-    Tp::ChannelRequestHints hints;
-    hints.setHint(QLatin1String("org.freedesktop.Telepathy.ChannelRequest"),
-                  QLatin1String("DelegateToPreferredHandler"),
-                  QVariant(true));
-
     if (match.selectedAction() == action(QLatin1String("start-text-chat"))) {
-
-        account->ensureTextChat(contact,
-                                QDateTime::currentDateTime(),
-                                QLatin1String("org.freedesktop.Telepathy.Client.KTp.TextUi"),
-                                hints);
-
+        KTp::Actions::startChat(account, contact);
     } else if (match.selectedAction() == action(QLatin1String("start-audio-call"))) {
-
-        account->ensureAudioCall(contact,
-                QLatin1String("audio"),
-                QDateTime::currentDateTime(),
-                QLatin1String("org.freedesktop.Telepathy.Client.KTp.CallUi"));
-
+        KTp::Actions::startAudioCall(account, contact);
     } else if (match.selectedAction() == action("start-video-call")) {
-
-        account->ensureAudioVideoCall(contact,
-                QLatin1String("audio"), QLatin1String("video"),
-                QDateTime::currentDateTime(),
-                QLatin1String("org.freedesktop.Telepathy.Client.KTp.CallUi"));
-
+        KTp::Actions::startAudioVideoCall(account, contact);
     } else if (match.selectedAction() == action("start-file-transfer")) {
 
-        QStringList filenames = KFileDialog::getOpenFileNames(
+      QStringList filenames = KFileDialog::getOpenFileNames(
                                     KUrl("kfiledialog:///FileTransferLastDirectory"),
                                     QString(),
                                     0,
@@ -266,60 +244,45 @@ void ContactRunner::run(const Plasma::RunnerContext &context, const Plasma::Quer
             return;
         }
 
-        foreach (const QString &filename, filenames) {
-            Tp::FileTransferChannelCreationProperties properties(
-                filename, KMimeType::findByFileContent(filename)->name());
-
-            account->createFileTransfer(contact,
-                                        properties,
-                                        QDateTime::currentDateTime(),
-                                        QLatin1String("org.freedesktop.Telepathy.Client.KTp.FileTransfer"));
+        Q_FOREACH (const QString &filename, filenames) {
+            KTp::Actions::startFileTransfer(account, contact, filename);
         }
 
     } else if (match.selectedAction() == action("start-desktop-sharing")) {
-
-        account->createStreamTube(contact,
-                                  QLatin1String("rfb"),
-                                  QDateTime::currentDateTime(),
-                                  QLatin1String("org.freedesktop.Telepathy.Client.krfb_rfb_handler"));
-
+        KTp::Actions::startDesktopSharing(account, contact);
     } else if (match.selectedAction() == action(QLatin1String("show-log-viewer"))) {
-
-	/* Use "--" so that KCmdLineArgs does not try to parse UIDs starting with "-" as arguments */
-        KToolInvocation::kdeinitExec(QLatin1String("ktp-log-viewer"),
-                                     QStringList() << QLatin1String("--") << account->uniqueIdentifier() << contact->id());
-
+        KTp::Actions::openLogViewer(account, contact);
     }
 }
 
-bool ContactRunner::hasCapability(const QModelIndex &contact, AccountsFilterModel::CapabilityFilterFlag capability) const
+bool ContactRunner::hasCapability(const Tp::ContactPtr &contact, Capability capability) const
 {
-    if (capability == AccountsFilterModel::DoNotFilterByCapability) {
+    if (capability == AllCapabilitites) {
         return true;
     }
 
-    if ((capability == AccountsFilterModel::FilterByTextChatCapability) &&
-        contact.data(AccountsModel::TextChatCapabilityRole).toBool()) {
+    if ((capability == TextChatCapability) &&
+        contact->capabilities().textChats()) {
         return true;
     }
 
-    if ((capability == AccountsFilterModel::FilterByAudioCallCapability) &&
-        contact.data(AccountsModel::AudioCallCapabilityRole).toBool()) {
+    if ((capability == AudioCallCapability) &&
+        contact->capabilities().audioCalls()) {
         return true;
     }
 
-    if ((capability == AccountsFilterModel::FilterByVideoCallCapability) &&
-        contact.data(AccountsModel::VideoCallCapabilityRole).toBool()) {
+    if ((capability == VideoCallCapability) &&
+        contact->capabilities().videoCalls()) {
         return true;
     }
 
-    if ((capability == AccountsFilterModel::FilterByFileTransferCapability) &&
-        contact.data(AccountsModel::FileTransferCapabilityRole).toBool()) {
+    if ((capability == FileTransferCapability) &&
+        contact->capabilities().fileTransfers()) {
         return true;
     }
 
-    if ((capability == AccountsFilterModel::FilterByDesktopSharingCapability) &&
-        contact.data(AccountsModel::DesktopSharingCapabilityRole).toBool()) {
+    if ((capability == DesktopSharingCapability) &&
+        contact->capabilities().streamTubes(QLatin1String("org.freedesktop.Telepathy.Client.krfb_rfb_handler"))) {
         return true;
     }
 
@@ -332,76 +295,66 @@ void ContactRunner::matchContacts(Plasma::RunnerContext &context)
 
     QAction *defaultAction;
     QString contactQuery;
-    AccountsFilterModel::CapabilityFilterFlag filterFlag;
+    Capability capability;
     if (term.startsWith(QLatin1String("chat "), Qt::CaseInsensitive)) {
         defaultAction = action(QLatin1String("start-text-chat"));
-        filterFlag = AccountsFilterModel::FilterByTextChatCapability;
+        capability = TextChatCapability;
         contactQuery = term.mid(QString("chat").length()).trimmed();
     } else if (term.startsWith(QLatin1String("audiocall "), Qt::CaseInsensitive)) {
         defaultAction = action(QLatin1String("start-audio-call"));
-        filterFlag = AccountsFilterModel::FilterByAudioCallCapability;
+        capability = AudioCallCapability;
         contactQuery = term.mid(QString("audiocall").length()).trimmed();
     } else if (term.startsWith(QLatin1String("videocall "), Qt::CaseInsensitive)) {
         defaultAction = action(QLatin1String("start-video-call"));
-        filterFlag = AccountsFilterModel::FilterByVideoCallCapability;
+        capability = VideoCallCapability;
         contactQuery = term.mid(QString("videocall").length()).trimmed();
     } else if (term.startsWith(QLatin1String("sendfile "), Qt::CaseInsensitive)) {
         defaultAction = action(QLatin1String("start-file-transfer"));
-        filterFlag = AccountsFilterModel::FilterByFileTransferCapability;
+        capability = FileTransferCapability;
         contactQuery = term.mid(QString("sendfile").length()).trimmed();
     } else if (term.startsWith(QLatin1String("sharedesktop "), Qt::CaseInsensitive)) {
         defaultAction = action(QLatin1String("start-desktop-sharing"));
-        filterFlag = AccountsFilterModel::FilterByDesktopSharingCapability;
+        capability = DesktopSharingCapability;
         contactQuery = term.mid(QString("sharedesktop").length()).trimmed();
     } else if (term.startsWith(QLatin1String("log "), Qt::CaseInsensitive)) {
         defaultAction = action(QLatin1String("show-log-viewer"));
-        filterFlag = AccountsFilterModel::DoNotFilterByCapability;
+        capability = AllCapabilitites;
         contactQuery = term.mid(QString("log").length()).trimmed();
     } else {
         defaultAction = action(QLatin1String("start-text-chat"));
-        filterFlag = AccountsFilterModel::DoNotFilterByCapability;
+        capability = AllCapabilitites;
         contactQuery = term;
     }
 
-    int accountsCnt = m_accountsModel->rowCount();
-    for (int i = 0; (i < accountsCnt) && context.isValid(); i++) {
+    Q_FOREACH (const Tp::AccountPtr &account, m_accountManager->allAccounts()) {
 
-        QModelIndex accountIndex = m_accountsModel->index(i, 0);
+        if (account->connection().isNull() || account->connection()->contactManager()->state() != Tp::ContactListStateSuccess) {
+            continue;
+        }
 
-        int contactsCount = m_accountsModel->rowCount(accountIndex);
-        for (int j = 0; (j < contactsCount) && context.isValid(); j++) {
+        Q_FOREACH (const Tp::ContactPtr &contact, account->connection()->contactManager()->allKnownContacts()) {
 
             Plasma::QueryMatch match(this);
             qreal relevance = 0.1;
 
-            QModelIndex contactIndex = m_accountsModel->index(j, 0, accountIndex);
-
-            if (!hasCapability(contactIndex, filterFlag)) {
+            if (!hasCapability(contact, capability)) {
                 continue;
             }
 
-            QString name = contactIndex.data(AccountsModel::AliasRole).toString();
+            const QString &name = contact->alias();
             if (!name.contains(contactQuery, Qt::CaseInsensitive)) {
                 continue;
             }
 
-            AccountsModelItem *accountItem = accountIndex.data(AccountsModel::ItemRole).value< AccountsModelItem* >();
-            ContactModelItem *contactItem = contactIndex.data(AccountsModel::ItemRole).value< ContactModelItem* >();
-            if (!accountItem || !contactItem) {
-                continue;
-            }
-
-            /* Store AccountsModelItem and ContactsModelItem as the data of match so that it can
-             * be retrieved quickly later */
             MatchInfo data;
-            data.account = accountIndex.data(AccountsModel::ItemRole).value< AccountsModelItem* >()->account();
-            data.contact = contactIndex.data(AccountsModel::ItemRole).value< ContactModelItem* >()->contact();
+            data.account = account;
+            data.contact = contact;
             match.setData(qVariantFromValue(data));
 
-            match.setText(name + QLatin1String(" (") +  accountIndex.data(AccountsModel::DisplayNameRole).toString() + ')');
+            match.setText(name + QLatin1String(" (") +  account->displayName() + ')');
             match.setType(Plasma::QueryMatch::ExactMatch);
 
-            KTp::Presence presence = contactIndex.data(AccountsModel::PresenceRole).value< KTp::Presence >();
+            KTp::Presence presence(contact->presence());
             switch (presence.type()) {
             case Tp::ConnectionPresenceTypeAvailable:
                 relevance *= 10;
@@ -424,7 +377,7 @@ void ContactRunner::matchContacts(Plasma::RunnerContext &context)
                 break;
             }
 
-            QString iconFile = contactIndex.data(AccountsModel::AvatarRole).toString();
+            QString iconFile = contact->avatarData().fileName;
             if (!iconFile.isEmpty() && QFile::exists(iconFile)) {
                 match.setIcon(QIcon(iconFile));
             } else {
